@@ -1,5 +1,5 @@
 /*
-** Copyright (C) 2003,2005 Erik de Castro Lopo <erikd@mega-nerd.com>
+** Copyright (C) 2003,2009 Erik de Castro Lopo <erikd@mega-nerd.com>
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU Lesser General Public License as published by
@@ -128,7 +128,7 @@ dither_init (SF_PRIVATE *psf, int mode)
 		if (pdither == NULL)
 			return SFE_MALLOC_FAILED ;
 
-		switch (psf->sf.format & SF_FORMAT_SUBMASK)
+		switch (SF_CODEC (psf->sf.format))
 		{	case SF_FORMAT_DOUBLE :
 			case SF_FORMAT_FLOAT :
 					pdither->read_int = psf->read_int ;
@@ -153,7 +153,7 @@ dither_init (SF_PRIVATE *psf, int mode)
 		if (pdither == NULL)
 			return SFE_MALLOC_FAILED ;
 
-		switch (psf->sf.format & SF_FORMAT_SUBMASK)
+		switch (SF_CODEC (psf->sf.format))
 		{	case SF_FORMAT_DOUBLE :
 			case SF_FORMAT_FLOAT :
 					pdither->write_int = psf->write_int ;
@@ -194,16 +194,14 @@ static void dither_float	(const float *in, float *out, int frames, int channels)
 static void dither_double	(const double *in, double *out, int frames, int channels) ;
 
 static sf_count_t
-dither_read_short (SF_PRIVATE *psf, short *ptr, sf_count_t len)
-{	psf = psf ;
-	ptr = ptr ;
+dither_read_short (SF_PRIVATE * UNUSED (psf), short * UNUSED (ptr), sf_count_t len)
+{
 	return len ;
 } /* dither_read_short */
 
 static sf_count_t
-dither_read_int (SF_PRIVATE *psf, int *ptr, sf_count_t len)
-{	psf = psf ;
-	ptr = ptr ;
+dither_read_int (SF_PRIVATE * UNUSED (psf), int * UNUSED (ptr), sf_count_t len)
+{
 	return len ;
 } /* dither_read_int */
 
@@ -221,7 +219,7 @@ dither_write_short	(SF_PRIVATE *psf, const short *ptr, sf_count_t len)
 		return 0 ;
 		} ;
 
-	switch (psf->sf.format & SF_FORMAT_SUBMASK)
+	switch (SF_CODEC (psf->sf.format))
 	{	case SF_FORMAT_PCM_S8 :
 		case SF_FORMAT_PCM_U8 :
 		case SF_FORMAT_DPCM_8 :
@@ -261,7 +259,7 @@ dither_write_int	(SF_PRIVATE *psf, const int *ptr, sf_count_t len)
 		return 0 ;
 		} ;
 
-	switch (psf->sf.format & SF_FORMAT_SUBMASK)
+	switch (SF_CODEC (psf->sf.format))
 	{	case SF_FORMAT_PCM_S8 :
 		case SF_FORMAT_PCM_U8 :
 		case SF_FORMAT_PCM_16 :
@@ -306,7 +304,7 @@ dither_write_float	(SF_PRIVATE *psf, const float *ptr, sf_count_t len)
 		return 0 ;
 		} ;
 
-	switch (psf->sf.format & SF_FORMAT_SUBMASK)
+	switch (SF_CODEC (psf->sf.format))
 	{	case SF_FORMAT_PCM_S8 :
 		case SF_FORMAT_PCM_U8 :
 		case SF_FORMAT_PCM_16 :
@@ -350,7 +348,7 @@ dither_write_double	(SF_PRIVATE *psf, const double *ptr, sf_count_t len)
 		return 0 ;
 		} ;
 
-	switch (psf->sf.format & SF_FORMAT_SUBMASK)
+	switch (SF_CODEC (psf->sf.format))
 	{	case SF_FORMAT_PCM_S8 :
 		case SF_FORMAT_PCM_U8 :
 		case SF_FORMAT_PCM_16 :
@@ -526,10 +524,4 @@ DO_NOT_USE_sf_dither_double (const SF_DITHER_INFO *dither, const double *in, dou
 } /* DO_NOT_USE_sf_dither_double */
 
 #endif
-/*
-** Do not edit or modify anything in this comment block.
-** The arch-tag line is a file identity tag for the GNU Arch 
-** revision control system.
-**
-** arch-tag: 673fad58-5314-421c-9144-9d54bfdf104c
-*/
+
