@@ -1,5 +1,5 @@
 /*
-** Copyright (C) 2002-2005 Erik de Castro Lopo <erikd@mega-nerd.com>
+** Copyright (C) 2002-2009 Erik de Castro Lopo <erikd@mega-nerd.com>
 ** Copyright (C) 2003 Ross Bencina <rbencina@iprimus.com.au>
 **
 ** This program is free software; you can redistribute it and/or modify
@@ -18,8 +18,18 @@
 */
 
 /*
-** This header file MUST be included before the others to ensure that
-** large file support is enabled.
+**	The file is split into three sections as follows:
+**		- The top section (USE_WINDOWS_API == 0) for Linux, Unix and MacOSX
+**			systems (including Cygwin).
+**		- The middle section (USE_WINDOWS_API == 1) for microsoft windows
+**			(including MinGW) using the native windows API.
+**		- A legacy windows section which attempted to work around grevious
+**			bugs in microsoft's POSIX implementation.
+*/
+
+/*
+**	The header file sfconfig.h MUST be included before the others to ensure
+**	that large file support is enabled correctly on Unix systems.
 */
 
 #include "sfconfig.h"
@@ -45,6 +55,15 @@
 
 #define	SENSIBLE_SIZE	(0x40000000)
 
+/*
+**	Neat solution to the Win32/OS2 binary file flage requirement.
+**	If O_BINARY isn't already defined by the inclusion of the system
+**	headers, set it to zero.
+*/
+#ifndef O_BINARY
+#define O_BINARY 0
+#endif
+
 static void psf_log_syserr (SF_PRIVATE *psf, int error) ;
 
 #if (USE_WINDOWS_API == 0)
@@ -108,7 +127,10 @@ psf_open_rsrc (SF_PRIVATE *psf, int open_mode)
 	psf->error = SFE_NO_ERROR ;
 	if ((psf->rsrcdes = psf_open_fd (psf->rsrcpath, open_mode)) >= 0)
 	{	psf->rsrclength = psf_get_filelen_fd (psf->rsrcdes) ;
-		return SFE_NO_ERROR ;
+		if (psf->rsrclength > 0 || (open_mode & SFM_WRITE))
+			return SFE_NO_ERROR ;
+		psf_close_fd (psf->rsrcdes) ;
+		psf->rsrcdes = -1 ;
 		} ;
 
 	if (psf->rsrcdes == - SFE_BAD_OPEN_MODE)
@@ -513,17 +535,17 @@ psf_open_fd (const char * pathname, int open_mode)
 
 	switch (open_mode)
 	{	case SFM_READ :
-				oflag = O_RDONLY ;
+				oflag = O_RDONLY | O_BINARY ;
 				mode = 0 ;
 				break ;
 
 		case SFM_WRITE :
-				oflag = O_WRONLY | O_CREAT | O_TRUNC ;
+				oflag = O_WRONLY | O_CREAT | O_TRUNC | O_BINARY ;
 				mode = S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH ;
 				break ;
 
 		case SFM_RDWR :
-				oflag = O_RDWR | O_CREAT ;
+				oflag = O_RDWR | O_CREAT | O_BINARY ;
 				mode = S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH ;
 				break ;
 
@@ -532,11 +554,6 @@ psf_open_fd (const char * pathname, int open_mode)
 				break ;
 		} ;
 
-#if OS_IS_WIN32
-	/* For Cygwin. */
-	oflag |= O_BINARY ;
-#endif
-
 	if (mode == 0)
 		fd = open (pathname, oflag) ;
 	else
@@ -575,10 +592,6 @@ psf_fsync (SF_PRIVATE *psf)
 #include <windows.h>
 #include <io.h>
 
-#ifndef HAVE_SSIZE_T
-typedef long ssize_t ;
-#endif
-
 static int psf_close_handle (HANDLE handle) ;
 static HANDLE psf_open_handle (const char * path, int mode) ;
 static sf_count_t psf_get_filelen_handle (HANDLE handle) ;
@@ -590,7 +603,7 @@ psf_fopen (SF_PRIVATE *psf, const char *pathname, int open_mode)
 	psf->hfile = psf_open_handle (pathname, open_mode) ;
 
 	if (psf->hfile == NULL)
-		psf_log_syserr (psf, errno) ;
+		psf_log_syserr (psf, GetLastError ()) ;
 
 	psf->mode = open_mode ;
 
@@ -610,7 +623,7 @@ psf_fclose (SF_PRIVATE *psf)
 		} ;
 
 	if ((retval = psf_close_handle (psf->hfile)) == -1)
-		psf_log_syserr (psf, errno) ;
+		psf_log_syserr (psf, GetLastError ()) ;
 
 	psf->hfile = NULL ;
 
@@ -655,7 +668,7 @@ psf_open_rsrc (SF_PRIVATE *psf, int open_mode)
 
 	/* No resource file found. */
 	if (psf->hrsrc == NULL)
-		psf_log_syserr (psf, errno) ;
+		psf_log_syserr (psf, GetLastError ()) ;
 
 	psf->hrsrc = NULL ;
 
@@ -845,7 +858,7 @@ psf_set_stdio (SF_PRIVATE *psf, int mode)
 /* USE_WINDOWS_API */ void
 psf_set_file (SF_PRIVATE *psf, int fd)
 {	HANDLE handle ;
-	long osfhandle ;
+	intptr_t osfhandle ;
 
 	osfhandle = _get_osfhandle (fd) ;
 	handle = (HANDLE) osfhandle ;
@@ -1148,10 +1161,6 @@ psf_ftruncate (SF_PRIVATE *psf, sf_count_t len)
 #include <io.h>
 #include <direct.h>
 
-#ifndef HAVE_SSIZE_T
-typedef long ssize_t ;
-#endif
-
 /* Win32 */ int
 psf_fopen (SF_PRIVATE *psf, const char *pathname, int open_mode)
 {	int oflag, mode ;
@@ -1515,10 +1524,3 @@ psf_log_syserr (SF_PRIVATE *psf, int error)
 
 #endif
 
-/*
-** Do not edit or modify anything in this comment block.
-** The arch-tag line is a file identity tag for the GNU Arch
-** revision control system.
-**
-** arch-tag: 749740d7-ecc7-47bd-8cf7-600f31d32e6d
-*/
