@@ -1,5 +1,5 @@
 /*
-** Copyright (C) 1999-2005 Erik de Castro Lopo <erikd@mega-nerd.com>
+** Copyright (C) 1999-2009 Erik de Castro Lopo <erikd@mega-nerd.com>
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU Lesser General Public License as published by
@@ -16,14 +16,15 @@
 ** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
+#include	"sfconfig.h"
+
 #include	<stdio.h>
 #include	<stdlib.h>
 #include	<string.h>
+#include	<math.h>
 
 #include	"sndfile.h"
-#include	"sfconfig.h"
 #include	"sfendian.h"
-#include	"float_cast.h"
 #include	"common.h"
 
 typedef struct IMA_ADPCM_PRIVATE_tag
@@ -82,8 +83,7 @@ static sf_count_t ima_write_d (SF_PRIVATE *psf, const double *ptr, sf_count_t le
 
 static sf_count_t	ima_seek	(SF_PRIVATE *psf, int mode, sf_count_t offset) ;
 
-static int	wav_w64_ima_close	(SF_PRIVATE *psf) ;
-static int	aiff_ima_close	(SF_PRIVATE *psf) ;
+static int	ima_close	(SF_PRIVATE *psf) ;
 
 static int wav_w64_ima_decode_block (SF_PRIVATE *psf, IMA_ADPCM_PRIVATE *pima) ;
 static int wav_w64_ima_encode_block (SF_PRIVATE *psf, IMA_ADPCM_PRIVATE *pima) ;
@@ -101,6 +101,11 @@ int
 wav_w64_ima_init (SF_PRIVATE *psf, int blockalign, int samplesperblock)
 {	int error ;
 
+	if (psf->codec_data != NULL)
+	{	psf_log_printf (psf, "*** psf->codec_data is not NULL.\n") ;
+		return SFE_INTERNAL ;
+		} ;
+
 	if (psf->mode == SFM_RDWR)
 		return SFE_BAD_MODE_RW ;
 
@@ -112,40 +117,12 @@ wav_w64_ima_init (SF_PRIVATE *psf, int blockalign, int samplesperblock)
 		if ((error = ima_writer_init (psf, blockalign)))
 			return error ;
 
+	psf->codec_close = ima_close ;
 	psf->seek = ima_seek ;
-	psf->close = wav_w64_ima_close ;
 
 	return 0 ;
 } /* wav_w64_ima_init */
 
-static int
-wav_w64_ima_close	(SF_PRIVATE *psf)
-{	IMA_ADPCM_PRIVATE *pima ;
-
-	if (! psf->fdata)
-		return 0 ;
-
-	pima = (IMA_ADPCM_PRIVATE*) psf->fdata ;
-
-	if (psf->mode == SFM_WRITE)
-	{	/*	If a block has been partially assembled, write it out
-		**	as the final block.
-		*/
-		if (pima->samplecount && pima->samplecount < pima->samplesperblock)
-			pima->encode_block (psf, pima) ;
-
-		psf->sf.frames = pima->samplesperblock * pima->blockcount / psf->sf.channels ;
-
-		if (psf->write_header)
-			psf->write_header (psf, SF_TRUE) ;
-		} ;
-
-	free (psf->fdata) ;
-	psf->fdata = NULL ;
-
-	return 0 ;
-} /* wav_w64_ima_close */
-
 int
 aiff_ima_init (SF_PRIVATE *psf, int blockalign, int samplesperblock)
 {	int error ;
@@ -161,38 +138,29 @@ aiff_ima_init (SF_PRIVATE *psf, int blockalign, int samplesperblock)
 		if ((error = ima_writer_init (psf, blockalign)))
 			return error ;
 
-	psf->seek = ima_seek ;
-	psf->close = aiff_ima_close ;
+	psf->codec_close = ima_close ;
 
 	return 0 ;
 } /* aiff_ima_init */
 
 static int
-aiff_ima_close	(SF_PRIVATE *psf)
+ima_close	(SF_PRIVATE *psf)
 {	IMA_ADPCM_PRIVATE *pima ;
 
-	if (! psf->fdata)
-		return 0 ;
-
-	pima = (IMA_ADPCM_PRIVATE*) psf->fdata ;
+	pima = (IMA_ADPCM_PRIVATE*) psf->codec_data ;
 
 	if (psf->mode == SFM_WRITE)
 	{	/*	If a block has been partially assembled, write it out
 		**	as the final block.
 		*/
-
 		if (pima->samplecount && pima->samplecount < pima->samplesperblock)
 			pima->encode_block (psf, pima) ;
 
-		if (psf->write_header)
-			psf->write_header (psf, SF_TRUE) ;
+		psf->sf.frames = pima->samplesperblock * pima->blockcount / psf->sf.channels ;
 		} ;
 
-	free (psf->fdata) ;
-	psf->fdata = NULL ;
-
 	return 0 ;
-} /* aiff_ima_close */
+} /* ima_close */
 
 /*============================================================================================
 ** IMA ADPCM Read Functions.
@@ -211,7 +179,7 @@ ima_reader_init (SF_PRIVATE *psf, int blockalign, int samplesperblock)
 	if (! (pima = malloc (pimasize)))
 		return SFE_MALLOC_FAILED ;
 
-	psf->fdata = (void*) pima ;
+	psf->codec_data = (void*) pima ;
 
 	memset (pima, 0, pimasize) ;
 
@@ -226,18 +194,25 @@ ima_reader_init (SF_PRIVATE *psf, int blockalign, int samplesperblock)
 	psf->datalength = (psf->dataend) ? psf->dataend - psf->dataoffset :
 							psf->filelength - psf->dataoffset ;
 
+    if (pima->blocksize == 0)
+	{	psf_log_printf (psf, "*** Error : pima->blocksize should not be zero.\n") ;
+		return SFE_INTERNAL ;
+		} ;
+
 	if (psf->datalength % pima->blocksize)
 		pima->blocks = psf->datalength / pima->blocksize + 1 ;
 	else
 		pima->blocks = psf->datalength / pima->blocksize ;
 
-	switch (psf->sf.format & SF_FORMAT_TYPEMASK)
+	switch (SF_CONTAINER (psf->sf.format))
 	{	case SF_FORMAT_WAV :
 		case SF_FORMAT_W64 :
 				count = 2 * (pima->blocksize - 4 * pima->channels) / pima->channels + 1 ;
 
 				if (pima->samplesperblock != count)
-					psf_log_printf (psf, "*** Warning : samplesperblock should be %d.\n", count) ;
+				{	psf_log_printf (psf, "*** Error : samplesperblock should be %d.\n", count) ;
+					return SFE_INTERNAL ;
+					} ;
 
 				pima->decode_block = wav_w64_ima_decode_block ;
 
@@ -657,9 +632,9 @@ ima_read_s (SF_PRIVATE *psf, short *ptr, sf_count_t len)
 	int			readcount, count ;
 	sf_count_t	total = 0 ;
 
-	if (! psf->fdata)
+	if (! psf->codec_data)
 		return 0 ;
-	pima = (IMA_ADPCM_PRIVATE*) psf->fdata ;
+	pima = (IMA_ADPCM_PRIVATE*) psf->codec_data ;
 
 	while (len > 0)
 	{	readcount = (len > 0x10000000) ? 0x10000000 : (int) len ;
@@ -682,9 +657,9 @@ ima_read_i (SF_PRIVATE *psf, int *ptr, sf_count_t len)
 	int			k, bufferlen, readcount, count ;
 	sf_count_t	total = 0 ;
 
-	if (! psf->fdata)
+	if (! psf->codec_data)
 		return 0 ;
-	pima = (IMA_ADPCM_PRIVATE*) psf->fdata ;
+	pima = (IMA_ADPCM_PRIVATE*) psf->codec_data ;
 
 	sptr = psf->u.sbuf ;
 	bufferlen = ARRAY_LEN (psf->u.sbuf) ;
@@ -710,9 +685,9 @@ ima_read_f (SF_PRIVATE *psf, float *ptr, sf_count_t len)
 	sf_count_t	total = 0 ;
 	float		normfact ;
 
-	if (! psf->fdata)
+	if (! psf->codec_data)
 		return 0 ;
-	pima = (IMA_ADPCM_PRIVATE*) psf->fdata ;
+	pima = (IMA_ADPCM_PRIVATE*) psf->codec_data ;
 
 	normfact = (psf->norm_float == SF_TRUE) ? 1.0 / ((float) 0x8000) : 1.0 ;
 
@@ -740,9 +715,9 @@ ima_read_d (SF_PRIVATE *psf, double *ptr, sf_count_t len)
 	sf_count_t	total = 0 ;
 	double 		normfact ;
 
-	if (! psf->fdata)
+	if (! psf->codec_data)
 		return 0 ;
-	pima = (IMA_ADPCM_PRIVATE*) psf->fdata ;
+	pima = (IMA_ADPCM_PRIVATE*) psf->codec_data ;
 
 	normfact = (psf->norm_double == SF_TRUE) ? 1.0 / ((double) 0x8000) : 1.0 ;
 
@@ -767,9 +742,9 @@ ima_seek (SF_PRIVATE *psf, int mode, sf_count_t offset)
 {	IMA_ADPCM_PRIVATE *pima ;
 	int			newblock, newsample ;
 
-	if (! psf->fdata)
+	if (! psf->codec_data)
 		return 0 ;
-	pima = (IMA_ADPCM_PRIVATE*) psf->fdata ;
+	pima = (IMA_ADPCM_PRIVATE*) psf->codec_data ;
 
 	if (psf->datalength < 0 || psf->dataoffset < 0)
 	{	psf->error = SFE_BAD_SEEK ;
@@ -827,7 +802,7 @@ ima_writer_init (SF_PRIVATE *psf, int blockalign)
 	if ((pima = calloc (1, pimasize)) == NULL)
 		return SFE_MALLOC_FAILED ;
 
-	psf->fdata = (void*) pima ;
+	psf->codec_data = (void*) pima ;
 
 	pima->channels			= psf->sf.channels ;
 	pima->blocksize			= blockalign ;
@@ -838,7 +813,7 @@ ima_writer_init (SF_PRIVATE *psf, int blockalign)
 
 	pima->samplecount = 0 ;
 
-	switch (psf->sf.format & SF_FORMAT_TYPEMASK)
+	switch (SF_CONTAINER (psf->sf.format))
 	{	case SF_FORMAT_WAV :
 		case SF_FORMAT_W64 :
 				pima->encode_block = wav_w64_ima_encode_block ;
@@ -893,9 +868,9 @@ ima_write_s (SF_PRIVATE *psf, const short *ptr, sf_count_t len)
 	int			writecount, count ;
 	sf_count_t	total = 0 ;
 
-	if (! psf->fdata)
+	if (! psf->codec_data)
 		return 0 ;
-	pima = (IMA_ADPCM_PRIVATE*) psf->fdata ;
+	pima = (IMA_ADPCM_PRIVATE*) psf->codec_data ;
 
 	while (len)
 	{	writecount = (len > 0x10000000) ? 0x10000000 : (int) len ;
@@ -918,9 +893,9 @@ ima_write_i (SF_PRIVATE *psf, const int *ptr, sf_count_t len)
 	int			k, bufferlen, writecount, count ;
 	sf_count_t	total = 0 ;
 
-	if (! psf->fdata)
+	if (! psf->codec_data)
 		return 0 ;
-	pima = (IMA_ADPCM_PRIVATE*) psf->fdata ;
+	pima = (IMA_ADPCM_PRIVATE*) psf->codec_data ;
 
 	sptr = psf->u.sbuf ;
 	bufferlen = ARRAY_LEN (psf->u.sbuf) ;
@@ -946,9 +921,9 @@ ima_write_f (SF_PRIVATE *psf, const float *ptr, sf_count_t len)
 	sf_count_t	total = 0 ;
 	float		normfact ;
 
-	if (! psf->fdata)
+	if (! psf->codec_data)
 		return 0 ;
-	pima = (IMA_ADPCM_PRIVATE*) psf->fdata ;
+	pima = (IMA_ADPCM_PRIVATE*) psf->codec_data ;
 
 	normfact = (psf->norm_float == SF_TRUE) ? (1.0 * 0x7FFF) : 1.0 ;
 
@@ -976,9 +951,9 @@ ima_write_d (SF_PRIVATE *psf, const double *ptr, sf_count_t len)
 	sf_count_t	total = 0 ;
 	double 		normfact ;
 
-	if (! psf->fdata)
+	if (! psf->codec_data)
 		return 0 ;
-	pima = (IMA_ADPCM_PRIVATE*) psf->fdata ;
+	pima = (IMA_ADPCM_PRIVATE*) psf->codec_data ;
 
 	normfact = (psf->norm_double == SF_TRUE) ? (1.0 * 0x7FFF) : 1.0 ;
 
@@ -998,11 +973,3 @@ ima_write_d (SF_PRIVATE *psf, const double *ptr, sf_count_t len)
 	return total ;
 } /* ima_write_d */
 
-
-/*
-** Do not edit or modify anything in this comment block.
-** The arch-tag line is a file identity tag for the GNU Arch 
-** revision control system.
-**
-** arch-tag: 75a54b82-ad18-4758-9933-64e00a7f24e0
-*/
