@@ -1,5 +1,5 @@
 /*
-** Copyright (C) 2002,2003 Erik de Castro Lopo <erikd@mega-nerd.com>
+** Copyright (C) 2002-2009 Erik de Castro Lopo <erikd@mega-nerd.com>
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU Lesser General Public License as published by
@@ -16,12 +16,12 @@
 ** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
-#include "config.h"
+#include "sfconfig.h"
 
 #include <stdio.h>
 #include <stdlib.h>
 
-#ifdef HAVE_UNISTD_H
+#if HAVE_UNISTD_H
 #include <unistd.h>
 #endif
 
@@ -30,10 +30,13 @@
 
 #include "common.h"
 
+#include "test_main.h"
+
 static void make_data (int *data, int len, int seed) ;
 
 static void file_open_test (const char *filename) ;
 static void file_read_write_test (const char *filename) ;
+static void file_truncate_test (const char *filename) ;
 
 static void test_open_or_die (SF_PRIVATE *psf, int linenum) ;
 static void test_close_or_die (SF_PRIVATE *psf, int linenum) ;
@@ -45,18 +48,6 @@ static void test_seek_or_die (SF_PRIVATE *psf, sf_count_t offset, int whence, sf
 
 
 
-int
-main (void)
-{	const char *filename = "file_io.dat" ;
-
-	file_open_test	(filename) ;
-	file_read_write_test	(filename) ;
-
-	unlink (filename) ;
-
-	return 0 ;
-} /* main */
-
 /*==============================================================================
 ** Actual test functions.
 */
@@ -66,8 +57,7 @@ file_open_test (const char *filename)
 {	SF_PRIVATE sf_data, *psf ;
 	int		error ;
 
-	printf ("    %-24s : ", "file_open_test") ;
-	fflush (stdout) ;
+	print_test_name ("Testing file open") ;
 
 	memset (&sf_data, 0, sizeof (sf_data)) ;
 	psf = &sf_data ;
@@ -127,8 +117,7 @@ file_read_write_test (const char *filename)
 	** write, test that psf_get_filelen() returns the new length.
 	*/
 
-	printf ("    %-24s : ", "file_write_test") ;
-	fflush (stdout) ;
+	print_test_name ("Testing file write") ;
 
 	memset (&sf_data, 0, sizeof (sf_data)) ;
 	psf = &sf_data ;
@@ -164,8 +153,7 @@ file_read_write_test (const char *filename)
 	** that the data is correct.
 	*/
 
-	printf ("    %-24s : ", "file_read_test") ;
-	fflush (stdout) ;
+	print_test_name ("Testing file read") ;
 
 	/* Test file open in write mode. */
 	psf->mode = SFM_READ ;
@@ -189,8 +177,7 @@ file_read_write_test (const char *filename)
 	** go back and check that all three blocks are correct.
 	*/
 
-	printf ("    %-24s : ", "file_seek_test") ;
-	fflush (stdout) ;
+	print_test_name ("Testing file seek") ;
 
 	/* Test file open in read/write mode. */
 	psf->mode = SFM_RDWR ;
@@ -228,8 +215,7 @@ file_read_write_test (const char *filename)
 	** the file will actually be a seek to the value given by psf->fileoffset.
 	*/
 
-	printf ("    %-24s : ", "file_offset_test") ;
-	fflush (stdout) ;
+	print_test_name ("Testing file offset") ;
 
 	/* Test file open in read/write mode. */
 	psf->mode = SFM_RDWR ;
@@ -274,6 +260,68 @@ file_read_write_test (const char *filename)
 	puts ("ok") ;
 } /* file_read_write_test */
 
+static void
+file_truncate_test (const char *filename)
+{	SF_PRIVATE sf_data, *psf ;
+	unsigned char buffer [256] ;
+	int k ;
+
+	/*
+	** Open a new file and write two blocks of data to the file. After each
+	** write, test that psf_get_filelen() returns the new length.
+	*/
+
+	print_test_name ("Testing file truncate") ;
+
+	memset (&sf_data, 0, sizeof (sf_data)) ;
+	memset (buffer, 0xEE, sizeof (buffer)) ;
+
+	psf = &sf_data ;
+	strncpy (psf->filename, filename, sizeof (psf->filename)) ;
+
+	/*
+	** Open the file write mode, write 0xEE data and then extend the file
+	** using truncate (the extended data should be 0x00).
+	*/
+	psf->mode = SFM_WRITE ;
+	test_open_or_die (psf, __LINE__) ;
+	test_write_or_die (psf, buffer, sizeof (buffer) / 2, 1, sizeof (buffer) / 2, __LINE__) ;
+	psf_ftruncate (psf, sizeof (buffer)) ;
+	test_close_or_die (psf, __LINE__) ;
+
+	/* Open the file in read mode and check the data. */
+	psf->mode = SFM_READ ;
+	test_open_or_die (psf, __LINE__) ;
+	test_read_or_die (psf, buffer, sizeof (buffer), 1, sizeof (buffer), __LINE__) ;
+	test_close_or_die (psf, __LINE__) ;
+
+	for (k = 0 ; k < SIGNED_SIZEOF (buffer) / 2 ; k++)
+		if (buffer [k] != 0xEE)
+		{	printf ("\n\nLine %d : buffer [%d] = %d (should be 0xEE)\n\n", __LINE__, k, buffer [k]) ;
+			exit (1) ;
+			} ;
+
+	for (k = SIGNED_SIZEOF (buffer) / 2 ; k < SIGNED_SIZEOF (buffer) ; k++)
+		if (buffer [k] != 0)
+		{	printf ("\n\nLine %d : buffer [%d] = %d (should be 0)\n\n", __LINE__, k, buffer [k]) ;
+			exit (1) ;
+			} ;
+
+	/* Open the file in read/write and shorten the file using truncate. */
+	psf->mode = SFM_RDWR ;
+	test_open_or_die (psf, __LINE__) ;
+	psf_ftruncate (psf, sizeof (buffer) / 4) ;
+	test_close_or_die (psf, __LINE__) ;
+
+	/* Check the file length. */
+	psf->mode = SFM_READ ;
+	test_open_or_die (psf, __LINE__) ;
+	test_seek_or_die (psf, 0, SEEK_END, SIGNED_SIZEOF (buffer) / 4, __LINE__) ;
+	test_close_or_die (psf, __LINE__) ;
+
+	puts ("ok") ;
+} /* file_truncate_test */
+
 /*==============================================================================
 ** Testing helper functions.
 */
@@ -295,8 +343,8 @@ static void
 test_close_or_die (SF_PRIVATE *psf, int linenum)
 {
 	psf_fclose (psf) ;
-	if (psf->filedes >= 0)
-	{	printf ("\n\nLine %d: psf->filedes should be < 0.\n\n", linenum) ;
+	if (psf_file_valid (psf))
+	{	printf ("\n\nLine %d: psf->filedes should not be valid.\n\n", linenum) ;
 		exit (1) ;
 		} ;
 
@@ -374,10 +422,15 @@ make_data (int *data, int len, int seed)
 		data [k] = rand () ;
 
 } /* make_data */
-/*
-** Do not edit or modify anything in this comment block.
-** The arch-tag line is a file identity tag for the GNU Arch 
-** revision control system.
-**
-** arch-tag: 0a21fb93-78dd-4f72-8338-4bca9e77b8c8
-*/
+
+void
+test_file_io (void)
+{	const char *filename = "file_io.dat" ;
+
+	file_open_test	(filename) ;
+	file_read_write_test	(filename) ;
+	file_truncate_test (filename) ;
+
+	unlink (filename) ;
+} /* main */
+
