@@ -1,5 +1,5 @@
 /*
-** Copyright (C) 2003-2006 Erik de Castro Lopo <erikd@mega-nerd.com>
+** Copyright (C) 2003-2009 Erik de Castro Lopo <erikd@mega-nerd.com>
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU Lesser General Public License as published by
@@ -23,11 +23,11 @@
 #include <fcntl.h>
 #include <string.h>
 #include <ctype.h>
+#include <math.h>
 
 #include "sndfile.h"
 #include "sfendian.h"
 #include "common.h"
-#include "float_cast.h"
 
 #define	MAX_XI_SAMPLES	16
 
@@ -68,22 +68,22 @@ xi_open	(SF_PRIVATE *psf)
 	if (psf->is_pipe)
 		return SFE_XI_NO_PIPE ;
 
-	if (psf->fdata)
-		pxi = psf->fdata ;
+	if (psf->codec_data)
+		pxi = psf->codec_data ;
 	else if ((pxi = calloc (1, sizeof (XI_PRIVATE))) == NULL)
 		return SFE_MALLOC_FAILED ;
 
-	psf->fdata = pxi ;
+	psf->codec_data = pxi ;
 
 	if (psf->mode == SFM_READ || (psf->mode == SFM_RDWR && psf->filelength > 0))
 	{	if ((error = xi_read_header (psf)))
 			return error ;
 		} ;
 
-	subformat = psf->sf.format & SF_FORMAT_SUBMASK ;
+	subformat = SF_CODEC (psf->sf.format) ;
 
 	if (psf->mode == SFM_WRITE || psf->mode == SFM_RDWR)
-	{	if ((psf->sf.format & SF_FORMAT_TYPEMASK) != SF_FORMAT_XI)
+	{	if ((SF_CONTAINER (psf->sf.format)) != SF_FORMAT_XI)
 			return	SFE_BAD_OPEN_FORMAT ;
 
 		psf->endian = SF_ENDIAN_LITTLE ;
@@ -128,10 +128,8 @@ xi_open	(SF_PRIVATE *psf)
 */
 
 static int
-xi_close	(SF_PRIVATE *psf)
+xi_close	(SF_PRIVATE * UNUSED (psf))
 {
-	psf = psf ;
-
 	return 0 ;
 } /* xi_close */
 
@@ -221,7 +219,7 @@ dpcm_seek (SF_PRIVATE *psf, int mode, sf_count_t offset)
 {	XI_PRIVATE	*pxi ;
 	int			total, bufferlen, len ;
 
-	if ((pxi = psf->fdata) == NULL)
+	if ((pxi = psf->codec_data) == NULL)
 		return SFE_INTERNAL ;
 
 	if (psf->datalength < 0 || psf->dataoffset < 0)
@@ -248,7 +246,7 @@ dpcm_seek (SF_PRIVATE *psf, int mode, sf_count_t offset)
 
 	psf_fseek (psf, psf->dataoffset, SEEK_SET) ;
 
-	if ((psf->sf.format & SF_FORMAT_SUBMASK) == SF_FORMAT_DPCM_16)
+	if ((SF_CODEC (psf->sf.format)) == SF_FORMAT_DPCM_16)
 	{	total = offset ;
 		bufferlen = ARRAY_LEN (psf->u.sbuf) ;
 		while (total > 0)
@@ -270,16 +268,14 @@ dpcm_seek (SF_PRIVATE *psf, int mode, sf_count_t offset)
 
 
 static int
-xi_write_header (SF_PRIVATE *psf, int calc_length)
+xi_write_header (SF_PRIVATE *psf, int UNUSED (calc_length))
 {	XI_PRIVATE	*pxi ;
 	sf_count_t	current ;
 	const char	*string ;
 
-	if ((pxi = psf->fdata) == NULL)
+	if ((pxi = psf->codec_data) == NULL)
 		return SFE_INTERNAL ;
 
-	calc_length = calc_length ; /* Avoid a compiler warning. */
-
 	current = psf_ftell (psf) ;
 
 	/* Reset the current header length to zero. */
@@ -496,7 +492,7 @@ dpcm_read_dsc2s (SF_PRIVATE *psf, short *ptr, sf_count_t len)
 	int			bufferlen, readcount ;
 	sf_count_t	total = 0 ;
 
-	if ((pxi = psf->fdata) == NULL)
+	if ((pxi = psf->codec_data) == NULL)
 		return 0 ;
 
 	bufferlen = ARRAY_LEN (psf->u.ucbuf) ;
@@ -521,7 +517,7 @@ dpcm_read_dsc2i (SF_PRIVATE *psf, int *ptr, sf_count_t len)
 	int			bufferlen, readcount ;
 	sf_count_t	total = 0 ;
 
-	if ((pxi = psf->fdata) == NULL)
+	if ((pxi = psf->codec_data) == NULL)
 		return 0 ;
 
 	bufferlen = ARRAY_LEN (psf->u.ucbuf) ;
@@ -547,7 +543,7 @@ dpcm_read_dsc2f (SF_PRIVATE *psf, float *ptr, sf_count_t len)
 	sf_count_t	total = 0 ;
 	float		normfact ;
 
-	if ((pxi = psf->fdata) == NULL)
+	if ((pxi = psf->codec_data) == NULL)
 		return 0 ;
 
 	normfact = (psf->norm_float == SF_TRUE) ? 1.0 / ((float) 0x80) : 1.0 ;
@@ -575,7 +571,7 @@ dpcm_read_dsc2d (SF_PRIVATE *psf, double *ptr, sf_count_t len)
 	sf_count_t	total = 0 ;
 	double		normfact ;
 
-	if ((pxi = psf->fdata) == NULL)
+	if ((pxi = psf->codec_data) == NULL)
 		return 0 ;
 
 	normfact = (psf->norm_double == SF_TRUE) ? 1.0 / ((double) 0x80) : 1.0 ;
@@ -605,7 +601,7 @@ dpcm_read_dles2s (SF_PRIVATE *psf, short *ptr, sf_count_t len)
 	int			bufferlen, readcount ;
 	sf_count_t	total = 0 ;
 
-	if ((pxi = psf->fdata) == NULL)
+	if ((pxi = psf->codec_data) == NULL)
 		return 0 ;
 
 	bufferlen = ARRAY_LEN (psf->u.sbuf) ;
@@ -630,7 +626,7 @@ dpcm_read_dles2i (SF_PRIVATE *psf, int *ptr, sf_count_t len)
 	int			bufferlen, readcount ;
 	sf_count_t	total = 0 ;
 
-	if ((pxi = psf->fdata) == NULL)
+	if ((pxi = psf->codec_data) == NULL)
 		return 0 ;
 
 	bufferlen = ARRAY_LEN (psf->u.sbuf) ;
@@ -656,7 +652,7 @@ dpcm_read_dles2f (SF_PRIVATE *psf, float *ptr, sf_count_t len)
 	sf_count_t	total = 0 ;
 	float		normfact ;
 
-	if ((pxi = psf->fdata) == NULL)
+	if ((pxi = psf->codec_data) == NULL)
 		return 0 ;
 
 	normfact = (psf->norm_float == SF_TRUE) ? 1.0 / ((float) 0x8000) : 1.0 ;
@@ -684,7 +680,7 @@ dpcm_read_dles2d (SF_PRIVATE *psf, double *ptr, sf_count_t len)
 	sf_count_t	total = 0 ;
 	double		normfact ;
 
-	if ((pxi = psf->fdata) == NULL)
+	if ((pxi = psf->codec_data) == NULL)
 		return 0 ;
 
 	normfact = (psf->norm_double == SF_TRUE) ? 1.0 / ((double) 0x8000) : 1.0 ;
@@ -725,7 +721,7 @@ dpcm_write_s2dsc (SF_PRIVATE *psf, const short *ptr, sf_count_t len)
 	int			bufferlen, writecount ;
 	sf_count_t	total = 0 ;
 
-	if ((pxi = psf->fdata) == NULL)
+	if ((pxi = psf->codec_data) == NULL)
 		return 0 ;
 
 	bufferlen = ARRAY_LEN (psf->u.ucbuf) ;
@@ -750,7 +746,7 @@ dpcm_write_i2dsc (SF_PRIVATE *psf, const int *ptr, sf_count_t len)
 	int			bufferlen, writecount ;
 	sf_count_t	total = 0 ;
 
-	if ((pxi = psf->fdata) == NULL)
+	if ((pxi = psf->codec_data) == NULL)
 		return 0 ;
 
 	bufferlen = ARRAY_LEN (psf->u.ucbuf) ;
@@ -776,7 +772,7 @@ dpcm_write_f2dsc (SF_PRIVATE *psf, const float *ptr, sf_count_t len)
 	sf_count_t	total = 0 ;
 	float		normfact ;
 
-	if ((pxi = psf->fdata) == NULL)
+	if ((pxi = psf->codec_data) == NULL)
 		return 0 ;
 
 	normfact = (psf->norm_float == SF_TRUE) ? (1.0 * 0x7F) : 1.0 ;
@@ -804,7 +800,7 @@ dpcm_write_d2dsc (SF_PRIVATE *psf, const double *ptr, sf_count_t len)
 	sf_count_t	total = 0 ;
 	double		normfact ;
 
-	if ((pxi = psf->fdata) == NULL)
+	if ((pxi = psf->codec_data) == NULL)
 		return 0 ;
 
 	normfact = (psf->norm_double == SF_TRUE) ? (1.0 * 0x7F) : 1.0 ;
@@ -832,7 +828,7 @@ dpcm_write_s2dles (SF_PRIVATE *psf, const short *ptr, sf_count_t len)
 	int			bufferlen, writecount ;
 	sf_count_t	total = 0 ;
 
-	if ((pxi = psf->fdata) == NULL)
+	if ((pxi = psf->codec_data) == NULL)
 		return 0 ;
 
 	bufferlen = ARRAY_LEN (psf->u.sbuf) ;
@@ -857,7 +853,7 @@ dpcm_write_i2dles (SF_PRIVATE *psf, const int *ptr, sf_count_t len)
 	int			bufferlen, writecount ;
 	sf_count_t	total = 0 ;
 
-	if ((pxi = psf->fdata) == NULL)
+	if ((pxi = psf->codec_data) == NULL)
 		return 0 ;
 
 	bufferlen = ARRAY_LEN (psf->u.sbuf) ;
@@ -883,7 +879,7 @@ dpcm_write_f2dles (SF_PRIVATE *psf, const float *ptr, sf_count_t len)
 	sf_count_t	total = 0 ;
 	float		normfact ;
 
-	if ((pxi = psf->fdata) == NULL)
+	if ((pxi = psf->codec_data) == NULL)
 		return 0 ;
 
 	normfact = (psf->norm_float == SF_TRUE) ? (1.0 * 0x7FFF) : 1.0 ;
@@ -911,7 +907,7 @@ dpcm_write_d2dles (SF_PRIVATE *psf, const double *ptr, sf_count_t len)
 	sf_count_t	total = 0 ;
 	double		normfact ;
 
-	if ((pxi = psf->fdata) == NULL)
+	if ((pxi = psf->codec_data) == NULL)
 		return 0 ;
 
 	normfact = (psf->norm_double == SF_TRUE) ? (1.0 * 0x7FFF) : 1.0 ;
@@ -1195,10 +1191,3 @@ d2dles_array (XI_PRIVATE *pxi, const double *src, short *dest, int count, double
 	pxi->last_16 = last_val ;
 } /* d2dles_array */
 
-/*
-** Do not edit or modify anything in this comment block.
-** The arch-tag line is a file identity tag for the GNU Arch 
-** revision control system.
-**
-** arch-tag: 1ab2dbe0-29af-4d80-9c6f-cb21b67521bc
-*/
